@@ -545,7 +545,7 @@ sub pretty_address {
     # complaining about AF_INET6 when Socket6 is not available
 
     if( (defined $ipv6_addr_len) && (length $addr == $ipv6_addr_len)) {
-	($port,$addrunpack) = unpack_sockaddr_in6 ($addr);
+	($port,$addrunpack) = Socket6::unpack_sockaddr_in6 ($addr);
 	$addrstr = inet_ntop (AF_INET6(), $addrunpack);
     } else {
 	($port,$addrunpack) = unpack_sockaddr_in ($addr);
@@ -820,8 +820,8 @@ sub sa_equal_p ($$$) {
 	($p2,$a2) = unpack_sockaddr_in ($sa2);
     } elsif($this->{'sockfamily'} == AF_INET6()) {
 	# IPv6 addresses
-	($p1,$a1) = unpack_sockaddr_in6 ($sa1);
-	($p2,$a2) = unpack_sockaddr_in6 ($sa2);
+	($p1,$a1) = Socket6::unpack_sockaddr_in6 ($sa1);
+	($p2,$a2) = Socket6::unpack_sockaddr_in6 ($sa2);
     } else {
 	return 0;
     }
@@ -894,7 +894,7 @@ sub receive_trap {
     return undef unless $remote_addr;
 
     if( (defined $ipv6_addr_len) && (length $remote_addr == $ipv6_addr_len)) {
-	($port,$iaddr) = unpack_sockaddr_in6($remote_addr);
+	($port,$iaddr) = Socket6::unpack_sockaddr_in6($remote_addr);
     } else {
 	($port,$iaddr) = unpack_sockaddr_in($remote_addr);
     }
@@ -943,7 +943,7 @@ sub receive_request {
     return undef unless $remote_addr;
 
     if( (defined $ipv6_addr_len) && (length $remote_addr == $ipv6_addr_len)) {
-	($port,$iaddr) = unpack_sockaddr_in6($remote_addr);
+	($port,$iaddr) = Socket6::unpack_sockaddr_in6($remote_addr);
     } else {
 	($port,$iaddr) = unpack_sockaddr_in($remote_addr);
     }
