@@ -1,5 +1,5 @@
 /*
- MRTG 2.17.4  -- Rateup
+ MRTG 2.17.10  -- Rateup
  *********************
 
  Rateup is a fast add-on to the great MRTG Traffic monitor.  It makes
@@ -37,8 +37,9 @@
 #ifndef NETWARE
 #include "../config.h"
 #endif
-#include <unistd.h>
 #endif
+#include <unistd.h>
+#include <sys/types.h>
 #include <limits.h>
 #include <stdio.h>
 #include <time.h>
@@ -83,7 +84,7 @@
 #include <gd.h>
 #include <gdfonts.h>
 
-char *VERSION = "2.17.4";
+char *VERSION = "2.17.10";
 char *program, *router, *routerpath;
 int histvalid;
 
@@ -326,8 +327,7 @@ image (file, maxvi, maxvo, maxx, maxy, xscale, yscale, growright, step, bits,
   /* Declare the image */
   gdImagePtr graph, brush_out, brush_outm, brush_outp;
   /* Declare color indexes */
-  int i_light, i_dark, i_blank, i_major, i_in, i_out, i_grid, i_inm, i_outm;
-  int i_outp, i_outpg;
+  int i_light, i_dark, i_blank, i_major, i_in, i_grid, i_inm, i_outpg;
   /* Dotted style */
   int styleDotted[3];
   if ((graph_label = (char **) calloc (1, sizeof (char *) * maxx)) == NULL)
@@ -870,13 +870,10 @@ image (file, maxvi, maxvo, maxx, maxy, xscale, yscale, growright, step, bits,
      allocation for the brush is essential */
   i_major = gdImageColorAllocate (graph, c_major);
   i_in = gdImageColorAllocate (graph, col_in[0], col_in[1], col_in[2]);
-  i_out =
     gdImageColorAllocate (brush_out, col_out[0], col_out[1], col_out[2]);
   i_grid = gdImageColorAllocate (graph, c_grid);
   i_inm = gdImageColorAllocate (graph, col_inm[0], col_inm[1], col_inm[2]);
-  i_outm =
     gdImageColorAllocate (brush_outm, col_outm[0], col_outm[1], col_outm[2]);
-  i_outp =
     gdImageColorAllocate (brush_outp, col_outp[0], col_outp[1], col_outp[2]);
   i_outpg =
     gdImageColorAllocate (graph, col_outp[0], col_outp[1], col_outp[2]);
@@ -1132,9 +1129,13 @@ image (file, maxvi, maxvo, maxx, maxy, xscale, yscale, growright, step, bits,
   gdImageDestroy (brush_outp);
   free (lhist);
   free (graph_label);
-  if (kMG)
+  /* By commenting out the next 4 lines, short_si will leak exactly one copy of kMG, but
+     otherwise the kMG values for the weekly/monthly/yearly graphs are wrong.  This should
+     also fix the crash reported in GitHub issue #3. */
+  /* if (kMG) {
     free(short_si);
-
+    short_si = short_si_def;
+  } */
 
 #ifdef WIN32
   /* got to remove the target under win32
@@ -1247,9 +1248,8 @@ readhist (file)
       cur = last.time;
       x = histvalid = 0;
       hist = history;
-      while (!feof (fi))
+      while (fgets (buf, 256, fi) != NULL)
 	{
-	  fgets (buf, 256, fi);
 	  if (sscanf (buf, "" LLD " " LLD " " LLD " " LLD " " LLD "",
 		      &rd[0], &rd[1], &rd[2], &rd[3], &rd[4]) < 5)
 	    {
@@ -1276,8 +1276,8 @@ readhist (file)
 	  if (hist->time > cur)
 	    {
 	      fprintf (stderr,
-		       "%s, Rateup ERROR: %s found %s's log file was corrupt\n          or not in sorted order:\ntime: %lu.",
-		       bufftime, program, router, (unsigned long) hist->time);
+		       "%s, Rateup ERROR: %s found %s's log file was corrupt or not in sorted order: time: %ld.\n",
+		       bufftime, program, router, (long) hist->time);
 	      retcode = 2;
 	      break;
 	    }
@@ -1397,9 +1397,8 @@ update (in, out, abs_max, absupdate)
   if (now < last.time)
     {
       fprintf (stderr,
-	       "%s, Rateup ERROR: %s found that %s's log file time of %lu was greater than now (%lu)\nERROR: Let's not do the time warp, again. Logfile unchanged.\n",
-	       bufftime, program, router, (unsigned long) last.time,
-	       (unsigned long) now);
+	       "%s, Rateup ERROR: %s found that %s's log file time of %ld was greater than now (%ld)\nERROR: Let's not do the time warp, again. Logfile unchanged.\n",
+	       bufftime, program, router, (long) last.time, (long) now);
       return;
     }
   sprintf (buf, "%s.tmp_%lu", router,(unsigned long)getpid());
@@ -1452,7 +1451,7 @@ update (in, out, abs_max, absupdate)
 	    {
 	      inr = diff (in, last.in);
 	      if (inr < 0) {
-                if (inr > - (long long) 1 << 32) { 	/* wrapped 32-bit counter? */
+                if (inr > - ((long long) 1 << 32)) { 	/* wrapped 32-bit counter? */
   		    inr += (long long) 1 << 32;
                 }
                 else {
@@ -1499,7 +1498,7 @@ update (in, out, abs_max, absupdate)
 	    {
 	      outr = diff (out, last.out);
 	      if (outr < 0) {	/* wrapped  counter? */
-                if (outr > - (long long) 1 << 32) {
+                if (outr > - ((long long) 1 << 32)) {
   		    outr += (long long) 1 << 32;
                 }
                 else {
@@ -1552,7 +1551,7 @@ update (in, out, abs_max, absupdate)
     }
   if ((fo = fopen (buf, "w")) != NULL)
     {
-      fprintf (fo, "%lu %s %s\n", (unsigned long) now, in, out);
+      fprintf (fo, "%ld %s %s\n", (long) now, in, out);
       last.time = now;
       /* what is this good for? */
       /* gauge und absolute */
@@ -1563,11 +1562,13 @@ update (in, out, abs_max, absupdate)
 	}
       else
 	{
-	  strcpy (last.in, in);
-	  strcpy (last.out, out);
+	  strncpy (last.in, in, MAXL); 
+          last.in[MAXL-1]='\0';         
+	  strncpy (last.out, out,MAXL);
+          last.out[MAXL-1]='\0';
 	}
-      fprintf (fo, "%lu " LLD " " LLD " " LLD " " LLD "\n",
-	       (unsigned long) now, inrate, outrate, inrate, outrate);
+      fprintf (fo, "%ld " LLD " " LLD " " LLD " " LLD "\n",
+	       (long) now, inrate, outrate, inrate, outrate);
       nout = 1;
       hist->time = now;
       hist->in = inrate;
@@ -1587,9 +1588,9 @@ update (in, out, abs_max, absupdate)
       /* gobble up every shred of data we can get ... */
       if (plannow < now)
 	{
-	  NEXT ((unsigned long) (now - plannow));
-	  fprintf (fo, "%lu " LLD " " LLD " " LLD " " LLD "\n",
-		   (unsigned long) now, (long long) inr, (long long) outr,
+	  NEXT ((long) (now - plannow));
+	  fprintf (fo, "%ld " LLD " " LLD " " LLD " " LLD "\n",
+		   (long) now, (long long) inr, (long long) outr,
 		   (long long) inmax, (long long) outmax);
 	  hist->time = now;
 	  hist->in = inr;
@@ -1605,8 +1606,8 @@ update (in, out, abs_max, absupdate)
       for (x = 1; x < DAY_COUNT; x++)
 	{
 	  NEXT (DAY_SAMPLE);
-	  fprintf (fo, "%lu " LLD " " LLD " " LLD " " LLD "\n",
-		   (unsigned long) now, (long long) inr, (long long) outr,
+	  fprintf (fo, "%ld " LLD " " LLD " " LLD " " LLD "\n",
+		   (long) now, (long long) inr, (long long) outr,
 		   (long long) inmax, (long long) outmax);
 	  hist->time = now;
 	  hist->in = inr;
@@ -1624,9 +1625,9 @@ update (in, out, abs_max, absupdate)
 
       if (plannow < now)
 	{
-	  NEXT ((unsigned long) (now - plannow));
-	  fprintf (fo, "%lu " LLD " " LLD " " LLD " " LLD "\n",
-		   (unsigned long) now, (long long) inr, (long long) outr,
+	  NEXT ((long) (now - plannow));
+	  fprintf (fo, "%ld " LLD " " LLD " " LLD " " LLD "\n",
+		   (long) now, (long long) inr, (long long) outr,
 		   inmax, outmax);
 	  hist->time = now;
 	  hist->in = inr;
@@ -1642,8 +1643,8 @@ update (in, out, abs_max, absupdate)
       for (x = 0; x < WEEK_COUNT; x++)
 	{
 	  NEXT (WEEK_SAMPLE);
-	  fprintf (fo, "%lu " LLD " " LLD " " LLD " " LLD "\n",
-		   (unsigned long) now, (long long) inr, (long long) outr,
+	  fprintf (fo, "%ld " LLD " " LLD " " LLD " " LLD "\n",
+		   (long) now, (long long) inr, (long long) outr,
 		   inmax, outmax);
 	  hist->time = now;
 	  hist->in = inr;
@@ -1661,9 +1662,9 @@ update (in, out, abs_max, absupdate)
 
       if (plannow < now)
 	{
-	  NEXT ((unsigned long) (now - plannow));
-	  fprintf (fo, "%lu " LLD " " LLD " " LLD " " LLD "\n",
-		   (unsigned long) now, (long long) inr, (long long) outr,
+	  NEXT ((long) (now - plannow));
+	  fprintf (fo, "%ld " LLD " " LLD " " LLD " " LLD "\n",
+		   (long) now, (long long) inr, (long long) outr,
 		   inmax, outmax);
 	  hist->time = now;
 	  hist->in = inr;
@@ -1679,8 +1680,8 @@ update (in, out, abs_max, absupdate)
       for (x = 0; x < MONTH_COUNT; x++)
 	{
 	  NEXT (MONTH_SAMPLE);
-	  fprintf (fo, "%lu " LLD " " LLD " " LLD " " LLD "\n",
-		   (unsigned long) now, (long long) inr, (long long) outr,
+	  fprintf (fo, "%ld " LLD " " LLD " " LLD " " LLD "\n",
+		   (long) now, (long long) inr, (long long) outr,
 		   inmax, outmax);
 	  hist->time = now;
 	  hist->in = inr;
@@ -1698,9 +1699,9 @@ update (in, out, abs_max, absupdate)
 
       if (plannow < now)
 	{
-	  NEXT ((unsigned long) (now - plannow));
-	  fprintf (fo, "%lu " LLD " " LLD " " LLD " " LLD "\n",
-		   (unsigned long) now, (long long) inr, (long long) outr,
+	  NEXT ((long) (now - plannow));
+	  fprintf (fo, "%ld " LLD " " LLD " " LLD " " LLD "\n",
+		   (long) now, (long long) inr, (long long) outr,
 		   inmax, outmax);
 	  hist->time = now;
 	  hist->in = inr;
@@ -1716,8 +1717,8 @@ update (in, out, abs_max, absupdate)
       for (x = 0; x < YEAR_COUNT; x++)
 	{
 	  NEXT (YEAR_SAMPLE);
-	  fprintf (fo, "%lu " LLD " " LLD " " LLD " " LLD "\n",
-		   (unsigned long) now, (long long) inr, (long long) outr,
+	  fprintf (fo, "%ld " LLD " " LLD " " LLD " " LLD "\n",
+		   (long) now, (long long) inr, (long long) outr,
 		   inmax, outmax);
 	  hist->time = now;
 	  hist->in = inr;
@@ -2114,6 +2115,7 @@ main (argc, argv)
 	      history[x].in = rand () % atoi (argv[argi + 1]);
 	      history[x].out = rand () % atoi (argv[argi + 2]);
 	    }
+	  /* fallthrough */
 	case 'u':		/* Update file */
 	  if (argv[argi][1] == 'p')
 	    {
